<?php
/*  Copyright 2008  Michael J. Walker  (email : mike@englishmike.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Called when [az-index] is encountered in a blog page.  This is the 
 * main entry point for displaying the index.
 *
 * @param $content [az-index]
 * @return string to substitute with (i.e. the index page)
 */
function az_insert_index($params) {
    az_trace('fn:insert_index: '.count($params));
    extract(shortcode_atts(array('id' => 0), $params));
    return az_get_index_content($id);
}

/**
 * Build up the index page from the definition of the index and 
 * the contents of the included posts.
 *
 * @return index content to be displayed or an error message if the index doesn't exist
 */
function az_get_index_content($indexid) {
    az_trace('fn:get_index_content');
    if ($indexid != 0) {
        $index = new az_request();
        $count = $index->set_vars_from_table($indexid);
        if ($count > 0) {
            $items = az_get_index_items($index);
            $content = az_format_index($items, $index->cols, $index->options, az_current_url(), $index->perpage, 
                                               $index->headsep, $index->csssingle, $index->cssgroup, $index->customlinks);
        } else if ($count !== false) {
            $content = az_error_message('invalid_index', $indexid); 
        }
    } else {
        $content = az_error_message('invalid_shortcode');
    }
    return $content;
}

function az_error_message($id, $param = '', $blink = true) {
	$output = '<div style="color:red">'; 
	switch ($id) {
        case 'invalid_index': 
            $output .= '<br/>AZINDEX ERROR: There is no index with the id of "'.$param.'" available.';
            break; 
        case 'invalid_shortcode': 
            $output .= '<br/>AZINDEX ERROR: Invalid az-index short-code found. A non-zero <em>id</em> parameter must be specified.';
      	    break; 
	}
	if ($blink) { 
		$output .= '<br/>Please notify blog/site administration of the problem.';
	}
	$output .= '<br/>(Message from the AZIndex plugin)<br/></div>';
	return $output;
}

/**
 * Get the current page URL from the HTTP headers.
 */
function az_current_url() {
    $url = "http://" . $_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
    $pos = strpos($url, "pgno=") - 1;
    // If there is already a pgno parameter in the url, remove it.
    if ($pos > 0) {
    	$end = strpos($url, "&", $pos + 1);
    	if (!$end) {
    		// The pgno is at the end of the url, just truncate it. 
    	    $end = strlen($url); 
    	} else {
    		// The pgno is not the last parameter, so remove the pgno and the following & concatenator.
    	    $pos++;
    	    $end++;
    	}
    	$url = substr_replace($url, "", $pos, $end - $pos);
    }
    return $url;
}

/**
 * Format the index page to be displayed. 
 *
 * @param unknown_type $items items in the index
 * @param unknown_type $cols number of columns 
 * @param unknown_type $options current options set for the index
 * @param unknown_type $pagelink current index page URL
 * @param unknown_type $perpage number of items to be displayed per page
 * @param unknown_type $headsep the string used to separate headings from subheadings
 * @param unknown_type $csssingle custom css for non-grouped index items
 * @param unknown_type $cssgroup custom css for grouped index items
 * @return the formatted index page to be displayed
 */
function az_format_index($items, $cols, $options, $pagelink, $perpage, $headsep, $csssingle, $cssgroup, $customlinks) {
    az_trace('fn:format_content: options = '.$options);
    
    // Add carriage returns to the start of each entry to make it easier to read the source HTML.
    $cr = chr(10);
    
    // Query the option settings for the index.
    $striped = az_is_set($options, 'custom-css-striping');
    $grouped = az_is_set($options, 'group-subhead');
    $alphalinks = az_is_set($options, 'alpha-links');
    $addspaces = az_is_set($options, 'add-spaces');
    $multipage = az_is_set($options, 'multipage');
    
    // Fetch the correct stylesheet for the currently set options. 
    $output = az_get_stylesheet($options, $cols, $grouped ? $cssgroup : $csssingle);
    
    if (!empty($items)) {

        // Set the default separator if the user hasn't set one.
        if (empty($headsep)) {
            $headsep = ' - ';
        }
        
        // If this is a multipage index and we're not on the first page
        // then we need to make sure links for pages prior to the current
        // page are set up correctly. 
        if ($multipage && $perpage > 0) {
            $currentpage = intval($_GET['pgno']);
            if ($currentpage > 0) $currentpage--;
            $pagecount = ceil(count($items) / $perpage);
            $pagelinks = az_format_pagelinks($currentpage, $pagecount, $pagelink, $options);
            $first = $currentpage * $perpage;
            $last = $first + $perpage;
        } else {
            $first = 0;
            $perpage = count($items);
            $last = $perpage;
        }
        
        // Build alphalink info for pages before the current page.
        if ($alphalinks) {
            $indexchars = az_collect_links($indexchars, $items, 0, $first, $perpage, $prevhead);
        }
        
        // Calculate the number of items in each column.
        // Note: no account is taken for the size of the items, so
        // the columns will probably be uneven to some degree.
        $incol = intval($perpage / $cols + 1);
        
        // Fetch the previous heading if were not starting from the beginning.
        if ($first > 0) $prevhead = $items[$first - 1]['head'];
        for ($col = 0; $col < $cols; $col++) {                        
            $output .= '<div class="azindex"><ul>';
            $odd = false;
            for ($i = 0; $i < $incol; $i++) {
                $spacer = '';
                $charlink = '';
                
                // Fetch the next item.
                $curitem = $first + $col * $incol + $i;
                $item = $items[$curitem];
                if (!empty($item) && $curitem < $last) {
                    // Check to see if next entry starts with a different letter.
                    if (strncasecmp($item['head'], $prevhead, 1)) {
                        // If we need an alphalink, add it to the list.
                        if ($alphalinks) {
                            $char = strtoupper($item['head'][0]);
                            $indexchars[] = array('char' => $char, 'page' => intval($curitem / $perpage));
                            $charlink = ' id="char_'.ord($char).'"';
                        }
                        // If add-spaces set, then add the spacer.
                        if ($addspaces && $curitem > $first && $i > 0) {
                            $spacer = '<li class="spacer"></li>';
                        }
                    }

                    // Output the index item.
                    
                    // Check to see if the items with the same heading are grouped together.
                    // Note: if at the top of a column, always output the heading.
                    $link = get_permalink($item['id']);
                    if ($grouped) {
                        if ($item['head'] != $prevhead || $i == 0) {
                            if (!empty($prevhead)) {
                                $output .= '</li>'.$cr.$spacer;
                                if ($striped && $i > 0) {
                                    $odd = !$odd;
                                }
                            }
                            $output .= '<li'.$charlink.($odd ? ' class="azalt"' : '').'>'.'<span class="head">'.$item['fullhead'].'</span>';
                        }
                        $output .= '<span class="subhead"><a href="'.$link.'">'.$item['subhead'].'</a></span>';
                        if (!empty($item['desc'])) {
                            $output .= '<span class="desc">'.$item['desc'].'</span>';
                        }
                    } else {
                        $output .= $spacer.'<li'.$charlink.($odd ? ' class="azalt"' : '').'>';
                        $output .= '<a href="'.$link.'"><span class="head">';
                        $output .= $item['fullhead'].'</span>';
                        if (!empty($item['subhead'])) {
                            $output .= $headsep.'<span class="subhead">'.$item['subhead'].'</span></a>';
                        } else {
                            $output .= '</a>';
                        }
                        if (!empty($item['desc'])) {
                            $output .= '<span class="desc">'.$item['desc'].'</span>';
                        }
                        $output .= '</li>'.$cr;
                        if ($striped) {
                            $odd = !$odd;
                        }
                    }
                    $prevhead = $item['head'];
                }
            }
            if ($grouped) {
                $output .= '</li>'.$cr;
                if ($striped) {
                    $odd = !$odd;
                }
            }
            $output .= '</ul></div>';
        }
        
        // Reset the block formatting.
        $output .= '<div style="clear:both;"></div>';
        
        // Add the alphabetical links we have been collecting.
        $linkspacer = '<div class="azlinkspacer"></div>';
        if ($alphalinks) {
            $indexchars = az_collect_links($indexchars, $items, $last, count($items), $perpage, $prevhead);
            $output = $linkspacer.az_format_links($indexchars, $currentpage, $pagecount, $pagelink, $customlinks, $options).$output;
        }

        // Add the page links to the index page.
        if ($multipage && $perpage > 0) {
            $linkabove = az_is_set($options, 'multipage-links-above');
            $linkbelow = az_is_set($options, 'multipage-links-below');
            if ($linkabove || !($linkabove || $linkbelow)) {
                $output = $linkspacer.$pagelinks.$output;
            }
            if ($linkbelow) {
                $output .= $pagelinks.$linkspacer;
            }
        }
    }
    return $output;
}

/**
 * Collect the alphabetical links for the items in the index. Links are
 * created for the first item to begin with each letter.
 *
 * @param $indexchars the index characters we have already collected
 * @param $items the items in the index
 * @param $first start item 
 * @param $last end item
 * @param $perpage number of items per page
 * @param $prevhead the previous heading before the starting item
 * @return an array characters and what page of the index they are on 
 */
function az_collect_links($indexchars, $items, $first, $last, $perpage, $prevhead) {
    for ($i = $first; $i < $last; $i++) {
        $item = $items[$i];
        if (!empty($item)) {
            // Next entry starts with a different letter.
            if (strncasecmp($item['head'], $prevhead, 1)) {
                $char = strtoupper($item['head'][0]);
                $indexchars[] = array('char' => $char, 'page' => intval($i / $perpage));
            }
        }
        $prevhead = $item['head'];
    }
    return $indexchars;
}

/**
 * Add links to the top of the index page(s), if required.  Note that the order in which 
 * the links are added is the order of the charactes in the constant AZ_INDEXCHARS.  Any
 * character links which are not in this constant will be added to the front of the list.
 *
 * @param $indexchars characters in the index
 * @param $currentpage the current page of the index
 * @param $pagecount the number of pages in the index
 * @param $pagelink the URL of the main index page
 * @param $options specified options
 * @return the output containing the index. 
 */
function az_format_links($indexchars, $currentpage, $pagecount, $pagelink, $customlinks, $options) {
    
    $tworows = az_is_set($options, 'alpha-links-two-rows');
    $chars = az_is_set($options, 'custom-links') ? $customlinks : AZ_INDEXCHARS;
    $gap = '&nbsp;'.($tworows ? '&nbsp;' : '');
    $paged = az_is_set($options, 'multipage');
    $pgno = strpos($pagelink, '?') !== false ? '&pgno=' : '?pgno=';

    // Check to see if the unused characters are to be output along with the alphalinks.
    if (az_is_set($options, 'alpha-links-unused')) {
        // Print out the non-alphanumeric links, if any exist.  They could be at the beginning
        // or the end of the array, so we have to find them first ($start) then set a couple of variables
        // so that the split is at the right place ($sign) and the first alphalink is located ($c and $j)
        if (az_is_set($options, 'non-alpha-end')) {
        	for ($k = count($indexchars) - 1; $k >= 0 && strpos($chars, $indexchars[$k]['char']) === false; $k--);
        	$start = $k + 1;
        	$sign = -1;
        } else {
        	$start = 0;
        	$sign = 1;
        }
    	for ($j = $start, $c = 0; $j < count($indexchars) && strpos($chars, $indexchars[$j]['char']) === false; $j++, $c++) {
            if ($j > $start) {
                $nonalpha .= $gap;
            }
            $pg = (!$paged || $indexchars[$j]['page'] == $currentpage) ? '' : $pagelink.$pgno.($indexchars[$j]['page'] + 1);
            $nonalpha .= '<a href="'.$pg.'#char_'.ord($indexchars[$j]['char']).'">'.$indexchars[$j]['char'].'</a>';
        }
        $half = (strlen($chars) - ($c * $sign)) / 2;
        $j = ($sign > 0) ? $c : 0;
        // Output the rest of the alphalinks, including unused characters and on two rows, if specified.
        for ($i = 0; $i < strlen($chars); $i++) {
            if ($tworows && $i >= $half) {
                $output .= '<br/>';
                $half *= 3;
            } else if ($i > 0 || $j > 0) {
                $output .= $gap;
            }
            if ($j < count($indexchars) && $chars[$i] == $indexchars[$j]['char']) {
                $pg = (!$paged || $indexchars[$j]['page'] == $currentpage) ? '' : $pagelink.$pgno.($indexchars[$j]['page'] + 1);
                $output .= '<a href="'.$pg.'#char_'.ord($chars[$i]).'">'.$chars[$i].'</a>';
                $j++;        
            } else {
                $output .= $chars[$i];
            }
        }
        if (!empty($nonalpha)) {
            // If we have some non-alpha links, then put them in the right place.
            if (az_is_set($options, 'non-alpha-end')) {
                $output .= $gap.trim($nonalpha);
            } else {
                $output = $nonalpha.trim($output);
            }
        }
    } else {
        // Output the alphalinks, on two rows if specified.
        $half = count($indexchars) / 2;
        for ($i = 0; $i < count($indexchars); $i++) {
            if ($tworows && $i >= $half) {
                $output .= '<br/>';
                $half *= 3;
            } else if ($i > 0) {
                $output .= $gap;
            }
            $pg = (!$paged || $indexchars[$i]['page'] == $currentpage) ? '' : $pagelink.$pgno.($indexchars[$i]['page'] + 1);
            $output .= '<a href="'.$pg.'#char_'.ord($indexchars[$i]['char']).'">'.$indexchars[$i]['char'].'</a>';
        }
    }
    return '<div class="azlinks">'.$output.'</div>';
}

/**
 * Generate the page links for a multipage index.
 *
 * @param $currentpage index of current page
 * @param $pagecount total number of pages
 * @param $pagelink the index page URL
 * @param $options the current options
 * @return the formatted page links
 */
function az_format_pagelinks($currentpage, $pagecount, $pagelink, $options) {
    $pgno = strpos($pagelink, '?') !== false ? '&pgno=' : '?pgno=';
    $output .= $currentpage <= 0 ? '&lt;' : '<a href="'.$pagelink.$pgno.$currentpage.'">&lt;</a>';
    $output .= '&nbsp;';
    for ($i = 0; $i < $pagecount; $i++) {
        $output .= ($i == $currentpage ? ($i + 1) : '<a href="'.$pagelink.$pgno.($i + 1).'">'.($i + 1).'</a>').'&nbsp;';
    }
    $output .= $currentpage >= $pagecount - 1 ? '&gt;' : '<a href="'.$pagelink.$pgno.($currentpage + 2).'">&gt;</a>';
    return '<div class="azpagelinks">'.$output.'</div>';
}

/**
 * Get the stylesheet needed for the current style of index. 
 *
 * @param $options specified options
 * @param $cols number of columns in the index.  If cols is zero, then
 *              we are fetching the stylesheet for the admin panel 
 * @return output containing the stylesheet
 */
function az_get_stylesheet($options, $cols, $customcss) {

    // Add tidy carriage returns to the end of each line.
    $cr = chr(10);

    // Don't want the "style" tag if we're going to display it in the admin panel.
    if ($cols > 0) {
        $output = '<style type="text/css">'.$cr;
        $output .= '.azindex {width:'.intval(100 / $cols).'%;float:left;}';    
    }

    // If custom css is required, set it.
    if (az_is_set($options, 'custom-css')) {
       $content = $customcss; 
    }
    
    if (empty($content)) {
        if (az_is_set($options, 'group-subhead')) {
            $content = '.azindex .head {font-weight:bold}'.$cr
                      .'.azindex .subhead {clear:left; float:left; padding-left:10px;}'.$cr
                      .'.azindex .desc {clear:left; float:left; font-size:80%; padding-left:20px;}'.$cr;
        } else {
            $content = '.azindex .head {}'.$cr
                      .'.azindex .subhead {}'.$cr
                      .'.azindex .desc {float:left; font-size:80%; padding-left:10px;}'.$cr;
        }
    
        $content .= '.azindex {padding:20px 0 20px 0}'.$cr
                   .'.azindex ul {list-style:none; padding:0 5px 0 5px; margin:0;}'.$cr
                   .'.azindex ul li {clear:both; padding-top:5px;}'.$cr
                   .'.azindex ul li.azalt {float:left; width:100%; background-color:lightgray;}'.$cr
                   .'.azindex .spacer {height:20px;}'.$cr
                   .'.azlinks {text-align:center;}'.$cr
                   .'.azlinkspacer {height:20px;}'.$cr
                   .'.azpagelinks {text-align:center;}'.$cr;
    }

    $output .= $content;
    
    if ($cols > 0) {
        $output .='</style>';
    }
    return $output;
}

/**
 * Test to see if a specific option is set for the index.
 *
 * @param $options all the options set for the current index
 * @param $option the option to test for
 * @return true if the option is set for this index
 */
function az_is_set($options, $option) {
    return strpos($options, $option) !== false;
}

/**
 * Fetch the items to be added to the index from the blog's database.
 *
 * @param $index the parameters specificed for the index
 * @return items to be included in the index
 */
function az_get_index_items($index) {
    global $wpdb;
    az_trace('fn:get_index_items: pageid: '.$post->ID);

    $fields = 'ID, post_title, post_excerpt, post_author';
    $query = "SELECT DISTINCT $fields FROM $wpdb->posts";
    
    if (!empty($index->tagids)) {
        $query .= " INNER JOIN $wpdb->term_relationships AS rel1 ON ($wpdb->posts.ID = rel1.object_id)"
                 ." INNER JOIN $wpdb->term_taxonomy AS tax1 ON (rel1.term_taxonomy_id = tax1.term_taxonomy_id"
                                                ." AND tax1.taxonomy = 'post_tag' AND tax1.term_id IN ($index->tagids)) ";
    }
    
    if (!empty($index->catids)) {
        $query .= " INNER JOIN $wpdb->term_relationships AS rel2 ON ($wpdb->posts.ID = rel2.object_id)"
                 ." INNER JOIN $wpdb->term_taxonomy AS tax2 ON (rel2.term_taxonomy_id = tax2.term_taxonomy_id"
                                                ." AND tax2.taxonomy = 'category' AND tax2.term_id IN ($index->catids))";
    }
    $selection = "'post'";
    if (az_is_set($index->options, 'include-pages')) {
    	if (az_is_set($index->options, 'include-pages-exclude-posts')) {
    		$selection = "'page'";
    	} else {
    		$selection .= ",'page'";
    	}
    }
    $query .= " WHERE post_status = 'publish' AND post_type IN ($selection)";

    $posts = $wpdb->get_results($query);

    $ignorechars = az_get_ignorechars($index);
    // Build an array of items from the query results.
    foreach ($posts as $row) {
    	$head = az_get_post_item($row->ID, $row->post_title, $row->post_excerpt, $row->post_author, $index->head, $index->headkey);
    	// Remove any characters we want to ignore during the search.
    	$sorthead = empty($ignorechars) ? $head : ltrim($head, $ignorechars);
    	
    	// Don't add item to the index if there is no heading.
    	if (!empty($sorthead)) {
            $items[] = array('id' => $row->ID,
                             'head' => $sorthead,
                             'fullhead' => $head, 
                             'subhead' => az_get_post_item($row->ID, $row->post_title, $row->post_excerpt, $row->post_author, $index->subhead, $index->subheadkey),
                             'desc' => az_get_post_item($row->ID, $row->post_title, $row->post_excerpt, $row->post_author, $index->desc, $index->desckey),
                             );
    	}
    }                     

    // Sort the items into alphabetical order, as specified.
    if ($items) {
    	global $az_indexchars, $az_nonalphaend;
        $az_indexchars = az_is_set($index->options, 'custom-links') ? $index->customlinks : AZ_INDEXCHARS;
        // Determine where to put the non-alpha starting entries by setting a signed variable.
        $az_nonalphaend = az_is_set($index->options, 'non-alpha-end') ? -1 : 1;
        $comparefn = az_is_set($index->options, 'custom-sort') ? $index->customsort : 'az_compare';
        usort($items, $comparefn);
    }
    return $items;    
}

/**
 * Compare function for sorting the index into alphabetical order, first using the 
 * heading, then the subheading, then the description.  Non-alpha numbers will be 
 * sorted to the top of the list (could make this an option at some point). 
 *
 * @param $in1 first index item to compare
 * @param $in2 second index item to compare
 * @return comparison result
 */
function az_compare($in1, $in2) {
    $rc = strcmp($in1['head'], $in2['head']);
    if ($rc == 0) {
        $rc = strcasecmp($in1['subhead'], $in2['subhead']);            
        if ($rc == 0) {
            $rc = strcasecmp($in1['desc'], $in2['desc']);            
        }
    } else {
        if (!empty($in1['head']) && !empty($in2['head'])) {
            global $az_indexchars, $az_nonalphaend;
        	// Check to see if either heading starts with a non-alphanumeric character.
            $pos1 = strpos($az_indexchars, strtoupper($in1['head'][0]));
            $pos2 = strpos($az_indexchars, strtoupper($in2['head'][0]));            
            if ($pos1 === false && !($pos2 === false)) {
                $rc = -1 * $az_nonalphaend;
            } else if ($pos2 === false && !($pos1 === false)) {
                $rc = 1 * $az_nonalphaend;
            }
        } 
    }
    return $rc;
}

function az_get_ignorechars($index) {
	$chars =  az_is_set($index->options, 'ignore-chars') ? htmlspecialchars_decode($index->ignorechars, ENT_QUOTES) : null;
    if (!empty($chars)) {
    	// Add curly single quotes to the trim string.
        if (strpos($chars, "'") !== false) {
            $chars .= chr(145).chr(146);
        }
        // Add curly double quotes to the trim string.
        if (strpos($chars, '"') !== false) {
            $chars .= chr(147).chr(148);
        }
    }
    return $chars;
}

/**
 * Get the item for a post specified in the index as the heading,
 * subheading, or description. 
 *
 * @param $type type of item to retrieve
 * @param $key item key, if the item is a custom field
 * @return item's value
 */
function az_get_post_item($postid, $title, $excerpt, $author, $type, $key = false) {
    switch ($type) {
        case 'title':
            $item = $title;  
            break; 
        case 'excerpt':
            $item = $excerpt;
            break; 
        case 'author':
            $item =  get_author_name($author);
            break; 
        case 'custom':
            // Only the first custom field found with key is used.
            $item = get_post_custom_values($key, $postid);
            $item = $item[0];
            break; 
    }
    return $item;
}
?>